import os

from charmhelpers.core import hookenv
from charmhelpers.core.services.base import (
    manage_ports,
    ServiceManager,
    )
from charmhelpers.core.services import helpers

import actions
import turnip_helpers


def render_env_template(config, name):
    return helpers.render_template(
        source='envs/{}.j2'.format(name),
        target='{}/{}'.format(os.path.join(config['base_dir'], 'envs'), name),
        owner=config['code_user'],
        group=config['code_group'])


def manage():
    if hookenv.hook_name() in ('install', 'upgrade-charm'):
        actions.execd_preinstall('turnip')
        actions.install_packages('turnip')
        actions.create_users('turnip')
        actions.unpack_source('turnip')
        actions.install_python_packages('turnip')

    extra_requirements = []

    if hookenv.config()['storage_external']:
        # Request a volume from the storage subordinate, and ensure the
        # service doesn't try to start until it's present.
        for relid in hookenv.relation_ids('data'):
            hookenv.relation_set(relid, mountpoint=actions.DATA_DIR)
        extra_requirements.append(turnip_helpers.TurnipStorageRelation())

    config = hookenv.config()
    unit_name = hookenv.local_unit().replace('/', '-')
    config['nagios_hostname'] = '{}-{}'.format(
        config['nagios_context'], unit_name)
    manager = ServiceManager([
        {
            'service': 'turnip',
            'ports': actions.get_ports(),
            'provided_data': [turnip_helpers.TurnipApiRelation()],
            'required_data': [config] + extra_requirements,
            'data_ready': [
                actions.ensure_repo_store_writable,
                helpers.render_template(
                    source='turnip-packbackendserver.conf.j2',
                    target='/etc/init/turnip-packbackendserver.conf',
                    owner=config['code_user'],
                    group=config['code_group'],
                    perms=0o644),
                helpers.render_template(
                    source='turnip-virtserver.conf.j2',
                    target='/etc/init/turnip-virtserver.conf',
                    owner=config['code_user'],
                    group=config['code_group'],
                    perms=0o644),
                helpers.render_template(
                    source='turnip-packfrontendserver.conf.j2',
                    target='/etc/init/turnip-packfrontendserver.conf',
                    owner=config['code_user'],
                    group=config['code_group'],
                    perms=0o644),
                helpers.render_template(
                    source='turnip-sshserver.conf.j2',
                    target='/etc/init/turnip-sshserver.conf',
                    owner=config['code_user'],
                    group=config['code_group'],
                    perms=0o644),
                helpers.render_template(
                    source='turnip-httpserver.conf.j2',
                    target='/etc/init/turnip-httpserver.conf',
                    owner=config['code_user'],
                    group=config['code_group'],
                    perms=0o644),
                render_env_template(config, 'REPO_STORE'),
                render_env_template(config, 'TURNIP_LOG_DIR'),
                render_env_template(config, 'VIRTINFO_ENDPOINT'),
                render_env_template(config, 'AUTHENTICATION_ENDPOINT'),
                actions.write_ssh_keys,
                render_env_template(config, 'PRIVATE_SSH_KEY_PATH'),
                render_env_template(config, 'PUBLIC_SSH_KEY_PATH'),
                helpers.render_template(
                    source='sudoers-cgit.j2',
                    target='/etc/sudoers.d/turnip-cgit',
                    perms=0o440),
                helpers.render_template(
                    source='cgitwrap.j2',
                    target='/usr/local/bin/cgitwrap',
                    perms=0o755),
                render_env_template(config, 'CGIT_EXEC_PATH'),
                render_env_template(config, 'SITE_NAME'),
                helpers.render_template(
                    source='turnip-logrotate.j2',
                    target='/etc/logrotate.d/turnip',
                    perms=0o644),
                ],
            'start': [
                actions.turnip_service_restart,
                actions.publish_wsgi_relations,
                manage_ports,
                actions.publish_website_relations,
                ],
            'stop': [
                manage_ports,
                actions.turnip_service_stop,
                ]
        },
        {
            'service': 'turnip-monitoring',
            'required_data': [config],
            'data_ready': [
                actions.install_nrpe_scripts,
                helpers.render_template(
                    source='turnip-nrpe.j2',
                    target='/etc/nagios/nrpe.d/check_{}.cfg'.format(unit_name),
                ),
                helpers.render_template(
                    source='turnip-nagios.j2',
                    target='/var/lib/nagios/export/service__{}-{}.cfg'.format(
                        config['nagios_context'], unit_name),
                ),
            ],
            'data_lost': [
                helpers.render_template(
                    source='turnip-nrpe.j2',
                    target='/etc/nagios/nrpe.d/check_{}.cfg'.format(unit_name),
                ),
                helpers.render_template(
                    source='turnip-nagios.j2',
                    target='/var/lib/nagios/export/service__{}-{}.cfg'.format(
                        config['nagios_context'], unit_name),
                ),
            ],
            'start': [
                actions.nagios_restart,
                ],
        },
    ])
    manager.manage()
